@ECHO OFF
@CLS
@ECHO -----------STARTING %~nx0-----------
@ECHO.
@REM Set directory where this file resides as working directory
cd /d "%~dp0"
@REM Check on next line if the Path of the Directory from PicoScope is correct!!!
@SET PicoScope-Dir="%ProgramFiles(x86)%\Pico Technology\PicoScope6"
@REM We "SET" here the parameters passed on the command line.
SET FilePicoScopeSettings=0
SET FilePicoScopeSetUpPsmacro=0
SET SaveCSV=0
SET COUNTER=0
SET StartWithAllChannelsOFF=0
:Loop1
SET /A COUNTER += 1
SET TempString=%1 %2 %3 %4 %5 %6 %7 %8 %9
FOR /F "tokens=%COUNTER% delims= " %%A IN ('ECHO %TempString%') DO SET TempVar_A=%%A
ECHO %TempVar_A% | FIND /I ".pssettings" >NUL
IF %ERRORLEVEL% EQU 0 SET FilePicoScopeSettings=%TempVar_A%
ECHO %TempVar_A% | FIND /I ".psmacro" >NUL
IF %ERRORLEVEL% EQU 0 SET FilePicoScopeSetUpPsmacro=%TempVar_A%
ECHO %TempVar_A% | FIND /I "SaveCSV" >NUL
IF %ERRORLEVEL% EQU 0 SET SaveCSV=1
ECHO %TempVar_A% | FIND /I "StartWithAllChannelsOFF" >NUL
IF %ERRORLEVEL% EQU 0 SET StartWithAllChannelsOFF=1
IF %COUNTER% NEQ 9 GOTO Loop1
SET COUNTER=0
@REM Reset errorlevel to 0:
VERIFY > nul
:EndLoop1
@ECHO ------------STARTING PICOSCOPE-----------
@REM The "START /MIN /WAIT cmd /c" is used as a workaround for avoiding the PicoScope to crash when this batch file is called by third party sw. First we set all channels OFF. We are using the "Menu.Help.Items?" query to detect if an existing instance of PicoScope is running.
IF %StartWithAllChannelsOFF% EQU 0 ECHO Skipped start with all channels off!
IF %StartWithAllChannelsOFF% EQU 0 GOTO SKIP0
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a Channel.#0.Range.SelectedIndex=0 ^> temp.txt^)
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a Channel.#1.Range.SelectedIndex=0 ^> temp.txt^)
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a Channel.#2.Range.SelectedIndex=0 ^> temp.txt^)
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a Channel.#3.Range.SelectedIndex=0 ^> temp.txt^)
:SKIP0
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a Menu.Help.Items? ^> temp.txt^)
@ECHO ERRORLEVEL=%ERRORLEVEL%
@SET ERROR=%ERRORLEVEL%
@REM Before the first run of PicoScope a costumized "preferences.xml" is copied to the user AppData directory. This because some settings can't be set automaticly in an other way, for example the waveformbuffer amount. The subversion process like the LabView "System Exec.vi" waits (it won't exit) for the post-commit hook process to finish (in this case PicoScope.exe, we like to keep it running in background for speeding things up). It does so by creating some pipes for redirecting stdin and stderr (Handles), creating a process that *inherits* those handles and waiting for all handles being closed. Unfortunately in windows a child process inherits all handles (e.g. pipes) from its parent process if it is not created with some special flags (DETACHED_PROCESS) and bInheritHandles set to FALSE. Even more unfortunately the windows "start" command don't set these flags and therefore it is not possible to create a background job with "start" from a subversion hook script. So this is not a subversion issue. It is simply that "start" is the wrong tool to start a background job from a subversion hook. That's why in the next line we are using the tool RunDetached.exe. Many thanks to: Christian Rodemeyer https://svn.haxx.se/users/archive-2008-11/0301.shtml
IF %ERROR% NEQ 0 Copy /Y PicoScope_preferences.xml "%USERPROFILE%\AppData\Local\Pico Technology\64358d19-ef91-48bb-9ae0-3b4689d44d99\preferences.xml" && START /MIN RunDetached.exe %PicoScope-Dir:~0,-1%\PicoScope.exe"
@REM IF %ERRORLEVEL% NEQ 0 Copy /Y PicoScope_preferences.xml "%USERPROFILE%\AppData\Local\Pico Technology\64358d19-ef91-48bb-9ae0-3b4689d44d99\preferences.xml" && START /MIN cmd /c ^(%PicoScope-Dir%\PicoScope.exe^)
IF %ERROR% NEQ 0 GOTO Loop2
IF %ERROR% EQU 0 GOTO EndLoop3
:Loop2
SET /A COUNTER += 1
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a Run.Pressed=True ^> temp.txt^)
IF %COUNTER% EQU 30 GOTO ERROR
IF %ERRORLEVEL% NEQ 0 GOTO Loop2
@ECHO LoopCount: %COUNTER%
:EndLoop2
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a Run.Pressed=True ^> temp.txt^)
SET COUNTER=0
:Loop3
SET /A COUNTER += 1
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a Run.Pressed? ^> temp.txt^)
PING localhost -n 2 >NUL
TYPE temp.txt | FIND /I "True"
IF %COUNTER% EQU 20 GOTO ERROR
IF %ERRORLEVEL% NEQ 0 GOTO Loop3
:EndLoop3
@ECHO The PicoScope is now ready for to recieve commands! LoopCount: %COUNTER%
@REM
@REM If given by first parameter the PicoScope Settings File wil be opened
IF %FilePicoScopeSettings% EQU 0 ECHO Skipped open settings file!
IF %FilePicoScopeSettings% EQU 0 GOTO SKIP1
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a File.Open.IO=%FilePicoScopeSettings% ^> temp.txt^)
TYPE temp.txt
:SKIP1
@REM
@REM RUN MACRO FILE:
IF %FilePicoScopeSetUpPsmacro% EQU 0 ECHO Skipped run macro file!
IF %FilePicoScopeSetUpPsmacro% EQU 0 GOTO SKIP2
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a %FilePicoScopeSetUpPsmacro% ^> temp.txt^)
TYPE temp.txt
:SKIP2
@REM
@REM Save Waveform in FILE:
IF %SaveCSV% EQU 0 ECHO Skipped save waveform in CSV format!
IF %SaveCSV% EQU 0 GOTO SKIP3
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a File.SaveAs.IO="%~dp0PicoScope_Waveform.csv" ^> temp.txt^)
TYPE temp.txt
:SKIP3
@REM Read avalable measurements from PicoScope:
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a Measurements.CSV? ^> temp.txt^)
TYPE temp.txt
SET COUNTER=0
REM We check if PicoScope is ready loading the settingsfile, only if present
IF %FilePicoScopeSettings% EQU 0 GOTO EndLoop4
:Loop4
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a Run.Pressed=True ^> temp.txt^)
SET /A COUNTER += 1
START /MIN /WAIT cmd /c ^(%PicoScope-Dir%\PicoScope /a Run.Pressed? ^> temp.txt^)
PING localhost -n 2 >NUL
TYPE temp.txt | FIND /I "True"
IF %COUNTER% EQU 10 GOTO EndLoop4
IF %ERRORLEVEL% NEQ 0 GOTO Loop4
:EndLoop4
@DEL temp.txt > NUL 2>&1
@ECHO The PicoScope is now ready! LoopCount: %COUNTER%
EXIT /B 0
:ERROR
@REM Using the EXIT command with parameter /B to set the ERRORLEVEL=1
@DEL temp.txt > NUL 2>&1
@taskkill /f /im PicoScope.com >nul 2>nul
@taskkill /f /im PicoScope.exe >nul 2>nul
EXIT /B 1